@@ -1,5 +1,5 @@
 package Tree::RB::XS;
-$Tree::RB::XS::VERSION = '0.03';
+$Tree::RB::XS::VERSION = '0.05';
 # VERSION
 # ABSTRACT: Red/Black Tree implemented in C, with similar API to Tree::RB
 
@@ -25,7 +25,9 @@ our %EXPORT_TAGS= (
 
 sub new {
 	my $class= shift;
-	my %options= @_ == 1 && ref $_[0] eq 'CODE'? ( compare_fn => $_[0] ) : @_;
+	my %options= @_ == 1 && ref $_[0] eq 'HASH'? %{$_[0]}
+		: @_ == 1? ( compare_fn => $_[0] )
+		: @_;
 	my $self= bless \%options, $class;
 	$self->_init_tree(delete $self->{key_type}, delete $self->{compare_fn});
 	$self->allow_duplicates(1) if delete $self->{allow_duplicates};
@@ -110,6 +112,17 @@ sub Tree::RB::XS::Iter::clone {
 *STORE= *put;
 *CLEAR= *clear;
 
+sub hseek {
+	my ($self, $key, $opts)= @_;
+	if (@_ == 2 && ref $key eq 'HASH') {
+		$opts= $key;
+		$key= $opts->{'-key'};
+	}
+	my $reverse= $opts && $opts->{'-reverse'} || 0;
+	my $node= defined $key? $self->get_node($key, $reverse? GET_LE_LAST() : GET_GE()) : undef;
+	$self->_set_hashiter($node, $reverse);
+}
+
 
 *LUEQUAL= *GET_EQ;
 *LUGTEQ=  *GET_GE;
@@ -137,35 +150,51 @@ Tree::RB::XS - Red/Black Tree implemented in C, with similar API to Tree::RB
 B<NOTICE:> This module is very new and you should give it some thorough testing before
 trusting it with production data.
 
-  use Tree::RB::XS qw/ :key_type :get /;
+  use Tree::RB::XS qw/ :cmp :get /;
   
   my $tree= Tree::RB::XS->new;
-  $tree->put($_ => $_) for 'a'..'z';
-  say $tree->get('a');
-  $tree->delete('a');
-  say $tree->get('a', GET_GT); # finds 'b'
-  $tree->delete('a','z');
-  $tree->delete($tree->min, $tree->max);
+  $tree->put($_ => $_) for 'a'..'z';    # store key/value, overwrite
+  say $tree->get('a');                  # get value by key, or undef
+  $tree->delete('a');                   # delete by key
+  say $tree->get('a', GET_GT);          # find relative to the key, finds 'b'
+  $tree->delete('a','z');               # delete a range, inclusive
+  $tree->delete($tree->min_node);       # delete using nodes or iterators
+  $tree->clear;                         # efficiently delete all nodes
   
-  # optimize for integer comparisons
-  $tree= Tree::RB::XS->new(key_type => KEY_TYPE_INT);
+  $tree= Tree::RB::XS->new(CMP_INT);    # optimize for integer comparisons
   $tree->put(1 => "x");
 
-  # optimize for floating-point comparisons
-  $tree= Tree::RB::XS->new(key_type => 'float', allow_duplicates => 1);
+  $tree= Tree::RB::XS->new(
+    compare_fn => 'float',              # optimize for floating-point
+    allow_duplicates => 1
+  );
   $tree->put(rand() => 1) for 1..1000;
   $tree->delete(0, $tree->get_node_lt(.5));
-
-  # optimize for byte strings
-  $tree= Tree::RB::XS->new(key_type => KEY_TYPE_BSTR);
+  
+  $tree= Tree::RB::XS->new(CMP_MEMCMP); # optimize for byte strings
   $tree->put("test" => "x");
   
-  # inspect the node objects
-  say $tree->min->key;
+  say $tree->min->key;                  # inspect the node objects
   say $tree->nth(0)->key;
   my $node= $tree->min;
   my $next= $node->next;
   $node->prune;
+  
+  # iterators
+  
+  for (my $i= $tree->iter; !$i->done; $i->step) {
+    if ($i->key ...) { $i->value .... }
+  }
+  
+  my $i= $tree->rev_iter;
+  while (my $node= $i->next) {
+    $node->prune if $node->key =~ ...;
+  }
+  
+  my $i= $tree->iter;
+  while (my @batch= $i->next_values(100)) {
+    ...
+  }
 
 =head1 DESCRIPTION
 
@@ -174,7 +203,7 @@ other search trees on CPAN are optimized comparisons of keys (speed), C<< O(log
 node-by-index lookup (which allows the tree to act as an array), and the option to
 allow duplicate keys while preserving insertion order.
 
-The API is close but not identical to L<Tree::RB>.
+The API is almost a complete superset of L<Tree::RB> with a few differences:
 
 =over
 
@@ -189,10 +218,6 @@ C<resort> is not implemented (would be lots of effort, and unlikely to be needed
 
 =item *
 
-tie-hash interface and hseek are not implemented.  (not hard, but does anyone need it?)
-
-=item *
-
 Tree structure is not mutable via the attributes of Node, nor can nodes be created
 independent from a tree.
 
@@ -207,26 +232,19 @@ Many functions have official names changed, but aliases are provided for compati
 =head2 new
 
   my $tree= Tree::RB::XS->new( %OPTIONS );
-                     ...->new( \&compare_fn );
+                     ...->new( $compare_fn );
+
+If C<new> is given a single parameter, it is assumed to be the C<compare_fn>.
 
 Options:
 
 =over
 
-=item L</key_type>
-
-Choose an optimized storage for keys.  The default is L</KEY_TYPE_ANY> which stores
-whole perl scalars.  All the other types are faster, but perl scalars give the fewest
-surprises.  If importing constants is annoying, you can specify one of C<"any">,
-C<"int">, C<"float">, C<"bstr">, or C<"ustr">.
-
 =item L</compare_fn>
 
-Choose a custom key-compare function.  The default is C<CMP_PERL> which gives
-you the behavior of Perl's C<cmp> operator.
-You can supply a custom coderef, but avoid doing that if you are concerned about
-speed; consider pre-processing your keys instead, to be able to use one of the
-built-in compare functions.
+Choose a custom key-compare function.  This can be the ID of an optimized function,
+a coderef, or the name of one of the optimized IDs like "int" for C<CMP_INT>.
+See below for details.
 
 =item L</allow_duplicates>
 
@@ -237,43 +255,31 @@ Whether to allow two nodes with the same key.  Defaults to false.
 Whether to enable full compatibility with L<Tree::RB>'s list-context behavior for L</get>.
 Defaults to false.
 
+=item L</key_type>
+
+This is an internal detail that probably shouldn't have been exposed, and might be removed
+in the future.  Choose one of the optimized C<compare_fn> instead.
+
 =back
 
 =head1 ATTRIBUTES
 
-=head2 key_type
-
-The key-storage strategy used by the tree.  Read-only; pass as an option to
-the constructor.
-
-This is one of the following values: L</KEY_TYPE_ANY>, L</KEY_TYPE_INT>,
-L</KEY_TYPE_FLOAT>, L</KEY_TYPE_BSTR>, or L</KEY_TYPE_USTR>.
-See the description in EXPORTS for full details on each.
-
-Integers are of course the most efficient, followed by floats, followed by
-byte-strings and unicode-strings, followed by 'ANY' (which stores a whole
-perl scalar).  BSTR and USTR are both faster than operating on Perl scalars,
-but both save an internal copy of your key and might be a bad idea if your
-keys are extremely large and nodes are frequently added to the tree.
-(modern Perls can copy-on-write to avoid copying the key)
-
 =head2 compare_fn
 
 Specifies the function that compares keys.  Read-only; pass as an option to
 the constructor.
 
-This is one of: L</CMP_PERL>, L</CMP_INT>, L</CMP_FLOAT>, L</CMP_MEMCMP>,
-L</CMP_UTF8>, L</CMP_NUMSPLIT>, or a coderef.
+This is one of: L</CMP_PERL> (default), L</CMP_INT>, L</CMP_FLOAT>, L</CMP_MEMCMP>,
+L</CMP_UTF8>, L</CMP_NUMSPLIT>, or a coderef.  C<CMP_INT> and C<CMP_FLOAT> are the
+most efficient, and internally store the key as a number.  C<CMP_MEMCMP> and
+C<CMP_UTF8> copy the key into an internal buffer, and offer moderate speed gains
+over C<CMP_PERL>.  C<CMP_PERL> is Perl's own C<cmp> operator.
 
-If set to a perl coderef, it should take two parameters and return an integer
+If set to a coderef, it should take two parameters and return an integer
 indicating their order in the same manner as Perl's C<cmp>.
-Note that this forces C<< key_type => KEY_TYPE_ANY >>.
 Beware that using a custom coderef throws away most of the speed gains from using
-this XS variant over plain L<Tree::RB>.
-
-The default comparison is C<CMP_PERL> which is equivalent to Perl's own C<cmp>
-comparator.  If you specify a C<key_type> and not a C<compare_fn>, the default
-comparator depends on the C<key_type>.
+this XS variant over plain L<Tree::RB>.  If speed is important, try pre-processing
+your keys in a way that allows you to use one of the built-in ones.
 
 Patches welcome, for anyone who wants to expand the list of optimized built-in
 comparison functions.
@@ -293,6 +299,12 @@ method like 'get' change its behavior in list context.  So, by deault, this attr
 false and C<get> always returns one value.  But if you set this to true, C<get> changes in
 list context to also return the Node, like is done in L<Tree::RB/lookup>.
 
+=head2 key_type
+
+The key-storage strategy used by the tree.  Read-only; pass as an option to
+the constructor.  This is an implementation detail that may be removed in a future
+version.
+
 =head2 size
 
 Returns the number of elements in the tree.
@@ -402,12 +414,12 @@ node after all nodes of the same key, preserving the insertion order.
 Delete any node with a key identical to C<$key>, and return the number of nodes
 removed.  If you supply two keys (or two nodes) this will delete those nodes and
 all nodes inbetween; C<$key1> is searched with mode C<GET_GE> and C<$key2> is
-searched with mode C<GET_LE>, so the keys themselves do not need to be found in
+searched with mode C<GET_LE_LAST>, so the keys themselves do not need to be found in
 the tree.
 The keys (or nodes) must be given in ascending order, else no nodes are deleted.
 
 If you want to delete a range *exclusive* of one or both ends of the range, just
-use the C</get_node> method with the desired mode to look up each end of the nodes
+use the L</get_node> method with the desired mode to look up each end of the nodes
 that you do want removed.
 
 =head2 clear
@@ -423,13 +435,13 @@ the nodes without worrying about the tree structure or shifting iterators aside.
               ...->iter($from_key, $get_mode=GET_GE); # from get_node
               ...->iter($from_node);                  # from existing node
 
-Return an L<iterator object|ITERATOR OBJECTS> that traverses the tree from min to max,
+Return an L<iterator object|/ITERATOR OBJECTS> that traverses the tree from min to max,
 or from the key or node you provide up to max.
 
 =head2 rev_iter
 
 Like C<iter>, but the C<< ->next >> and C<< ->step >> methods walk backward to smaller key
-values, and the default C<$get_mode> is L</GET_GE_LAST>.
+values, and the default C<$get_mode> is L</GET_LE_LAST>.
 
 =head1 NODE OBJECTS
 
@@ -600,9 +612,9 @@ or you would lose elements)
 
 =item step
 
-    $iter->step;     # advance by one node
-	$iter->step(10); # advance by 10 nodes
-	$iter->step(-4); # back up 4 nodes
+  $iter->step;     # advance by one node
+  $iter->step(10); # advance by 10 nodes
+  $iter->step(-4); # back up 4 nodes
 
 This moves the iterator by one or more nodes in the forward or backward direction.
 For a reverse-iterator, positive numbers move toward the minimum key and negative numbers
@@ -612,7 +624,7 @@ node, the first node becomes the current node.
 
 =item delete
 
-Delete the "current node", return its value, and advance to the next node.
+Delete the current node, return its value, and advance to the next node.
 
   for (my $i= $tree->iter; !$i->done;) {
     if ($i->key =~ ...) {
@@ -642,48 +654,33 @@ This class implements the required methods needed for C<tie>:
   $hash{$_}= $_ for 1..10;
   delete $hash{3};
   $_ += 1 for values %hash;
+  tied(%hash)->hseek(5);
+  say each %hash;  # 5
 
 But you get better performance by using the tree's API directly.  This should only be used when
 you need to integrate with code that isn't aware of the tree.
 
-=head1 EXPORTS
-
-=head2 Key Types
-
-Export all with ':key_type';
-
 =over
 
-=item KEY_TYPE_ANY
+=item hseek
 
-This C<key_type> causes the tree to store whole Perl scalars for each node.
-Its default comparison function is Perl's own C<cmp> operator.
+  tied(%hash)->hseek( $key );
+             ->hseek({ -reverse => $bool });
+             ->hseek( $key, { -reverse => $bool });
 
-=item KEY_TYPE_INT
+This is a method of the tree, but only relevant to the tied hash interface.  It controls the
+behavior of the next call to C<< each %hash >> or C<< keys %hash >>, causing the first element
+to be the node at or after the C<$key>.  (or before, if you change to a reverse iterator)
 
-This C<key_type> causes the tree to store keys as Perl's integers,
-which are either 32-bit or 64-bit depending on how Perl was compiled.
-Its default comparison function puts the numbers in non-decreasing order.
-
-=item KEY_TYPE_FLOAT
-
-This C<key_type> causes the tree to store keys as Perl's floating point type,
-which are either 64-bit doubles or 80-bit long-doubles.
-Its default comparison function puts the numbers in non-decreasing order.
-
-=item KEY_TYPE_BSTR
-
-This C<key_type> causes the tree to store keys as byte strings.
-The default comparison function is the standard Libc C<memcmp>.
-
-=item KEY_TYPE_USTR
-
-Same as C<KEY_TYPE_BSTR> but reads the bytes from the supplied key as UTF-8 bytes.
-The default comparison function is also C<memcmp> even though this does not sort
-Unicode correctly.  (for correct unicode, use C<KEY_TYPE_ANY>, but it's slower...)
+This method differs from L<Tree::RB/hseek> in that C<Tree::RB> will change the logical first
+node of the iteration *indefinitely* such that repeated calls to C<keys> do not see any element
+less than C<$key>.  This C<hseek> only applies to the next iteration.  (which I'm guessing was
+th intent in Tree::RB?)
 
 =back
 
+=head1 EXPORTS
+
 =head2 Comparison Functions
 
 Export all with ':cmp'
@@ -750,6 +747,42 @@ C<memcmp>, else they are sorted with Perl's unicode-aware sort.
 
 =back
 
+=head2 Key Types
+
+Export all with ':key_type';
+
+=over
+
+=item KEY_TYPE_ANY
+
+This C<key_type> causes the tree to store whole Perl scalars for each node.
+Its default comparison function is Perl's own C<cmp> operator.
+
+=item KEY_TYPE_INT
+
+This C<key_type> causes the tree to store keys as Perl's integers,
+which are either 32-bit or 64-bit depending on how Perl was compiled.
+Its default comparison function puts the numbers in non-decreasing order.
+
+=item KEY_TYPE_FLOAT
+
+This C<key_type> causes the tree to store keys as Perl's floating point type,
+which are either 64-bit doubles or 80-bit long-doubles.
+Its default comparison function puts the numbers in non-decreasing order.
+
+=item KEY_TYPE_BSTR
+
+This C<key_type> causes the tree to store keys as byte strings.
+The default comparison function is the standard Libc C<memcmp>.
+
+=item KEY_TYPE_USTR
+
+Same as C<KEY_TYPE_BSTR> but reads the bytes from the supplied key as UTF-8 bytes.
+The default comparison function is also C<memcmp> even though this does not sort
+Unicode correctly.  (for correct unicode, use C<KEY_TYPE_ANY>, but it's slower...)
+
+=back
+
 =head2 Lookup Mode
 
 Export all with ':get'
@@ -812,26 +845,36 @@ Has alias C<LUPREV> to match Tree::RB.
 
 =back
 
-=head1 BUGS AND LIMITATIONS
+=head1 SEE ALSO
 
 =over
 
-=item *
+=item L<Tree::RB>
 
-Before Perl 5.14, there isn't any built-in utf8-aware case-sensitive compare function, so
-requests for C<CMP_UTF8> get changed to C<CMP_PERL> (with C<KEY_TYPE_ANY>) and C<CMP_NUMSPLIT>
-loses UTF8-awareness.
+The Red/Black module this one used as API inspiration.  The fastest pure-perl tree module on
+CPAN.  Implemented as blessed arrayrefs.
+
+=item L<AVLTree>
+
+Another XS-based tree module.  About 6%-70% slower than this one depending on whether you use
+coderef comparisons or optimized comparisons.
+
+=item L<Tree::AVL>
+
+An AVL tree implementation in pure perl.  The API is perhaps more convenient, with the ability
+to add your object to the tree with a callback that derives the key from that object.
+However, it runs significantly slower than Tree::RB.
 
 =back
 
+=head1 VERSION
+
+version 0.05
+
 =head1 AUTHOR
 
 Michael Conrad <mike@nrdvana.net>
 
-=head1 VERSION
-
-version 0.03
-
 =head1 COPYRIGHT AND LICENSE
 
 This software is copyright (c) 2021 by Michael Conrad.
